/*
* QuickFox Notes by Amin E (InBasic)
* eMail: inb.cor@gmail.com
*
* Source Code License: Mozilla Public License, version 1.1
*/
QuickFoxNotes.network = {
	_sendReq: function (me, url, method, header, doc, fun, parameters, errFun) {
		var req = new XMLHttpRequest();
		req.open(method, url, true);
		for (var i in header)
			req.setRequestHeader(header[i][0], header[i][1]);
		req.onreadystatechange = function (aEvt) {
			if (req.readyState == 4) {
				if(req.status == 200 || req.status == 201 || req.status == 204)
					fun.apply(me, [req].concat(parameters));
				else {
					if (errFun)
						fun.apply(me, [req]);
					else
						alert("Error occured:\n\n" + 
								"\tStatus: " + req.status +
								"\nMessage:\n\n" + req.responseText + 
								"\nHeader:\n\n" + req.getAllResponseHeaders());
				}
			}
		};
		req.send(doc);
	},
	/*
	<file>
		<entry>
			entry 1
		<entry>
		<entry>
			entry 2
		<entry>
	</file>
	
	var doc = document.implementation.createDocument("", "", null);
	_makeDoc(doc, doc,
		["file",[
		 ["entry", "entry 1"], 
		 ["entry", "entry 2"]
		]]
	);
	*/
	_makeDoc: function(doc, parent, arr) {
		var node
		if (typeof(arr[0]) == "string") {
			if (/@/.test(arr[0]))
				node = doc.createElementNS(arr[0].split("@")[1], arr[0].split("@")[0]);
			else
				node = doc.createElement(arr[0]);
		}
		else {
			for(i in arr)
				this._makeDoc(doc, parent, arr[i]);
			return;
		}

		if (typeof(arr[1]) == "string")
			node.textContent = arr[1];
		else
			this._makeDoc(doc, node, arr[1]);

		parent.appendChild(node)
		return node;
	},
	/**************************Google Docs***********************************/
	googleDocs: {
		token: null,
		
		getToken: function(username, password, fun, parameters, errFun, force) {
			if (this.token && !force) {
				fun.apply(QuickFoxNotes.network.googleDocs, [this.token].concat(parameters));
				return;
			}
			//http://code.google.com/apis/accounts/docs/AuthForInstalledApps.html
			var doc = "accountType=HOSTED_OR_GOOGLE" + 
				"&Email=" + username + 
				"&Passwd=" + password + 
				"&source=InBasic-QuickFoxNotes" + 
				"&scope=https://docs.google.com/feeds/%20https://spreadsheets.google.com/feeds/%20https://docs.googleusercontent.com/" + //Allow to read and write files
				"&service=writely" 	//http://code.google.com/apis/gdata/faq.html#clientlogin
			
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.googleDocs, "https://www.google.com/accounts/ClientLogin", "POST", 
				[['Content-Type', 'application/x-www-form-urlencoded']], doc, function(req){
					try {
						var Auth = req.responseText.match(/Auth=(.*)/)[1];
						this.token = Auth;
						fun.apply(QuickFoxNotes.network.googleDocs, [this.token].concat(parameters));
					}
					catch(e){
						errFun(req)
					}
				}, parameters, errFun);
		},

		createFile: function (token, url, name, data, fun, parameters) {
			var doc = document.implementation.createDocument("", "", null);
			QuickFoxNotes.network._makeDoc(doc, doc,
				["entry@http://www.w3.org/2005/Atom",[	//Add namespace to all sub elements too
				 ["category@http://www.w3.org/2005/Atom", ""], 
				 ["title@http://www.w3.org/2005/Atom", name]
				]]
			);
			var entry = doc.getElementsByTagName("entry")[0];
			entry.setAttribute("xmlns:docs", "http://schemas.google.com/docs/2007");
			var category = doc.getElementsByTagName("category")[0];
			category.setAttribute("scheme", "http://schemas.google.com/g/2005#kind");
			category.setAttribute("term", "http://schemas.google.com/docs/2007#document");
			
			var serializer = new XMLSerializer();
			doc = 
				'--END_OF_PART\n' +
				'Content-Type: application/atom+xml\n\n' +
				serializer.serializeToString(doc) +
				'\n\n--END_OF_PART\n' +
				'Content-Type: text/html\n\n' +
				'<pre>' + data + '</pre>' + '\n\n' +
				'--END_OF_PART--';

			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.googleDocs, url, "POST", 
				[["GData-Version", "3.0"],
				 ["Authorization", "GoogleLogin auth=" + token],
				 ["Content-Length", data.length],
				 ["Content-Type", "multipart/related; boundary=END_OF_PART"]], doc, function(req){
					try {
						var entry = req.responseXML.documentElement;
						var content = entry.getElementsByTagName("content")[0];
						var contentURL = content.getAttribute("src");
						var urls = entry.getElementsByTagName("link");
						var editURL;
						for (var j = 0; j < urls.length; j++){
							if (urls[j].getAttribute("rel") == "edit") {
								editURL = urls[j].getAttribute("href");
							}
							if (urls[j].getAttribute("rel") == "edit") {
								editURL = urls[j].getAttribute("href");
							}
						}

						fun.apply(QuickFoxNotes.network.googleDocs, [
							{
								contentURL: contentURL,
								editURL: editURL
							}].concat(parameters));
					}
					catch(e) {
						alert("netwrok>createFile\n\n" + e.message);
					}
				 }, parameters);
		},
		getList: function (token, fun, parameters, url, files) {	//url and files are only used by this function for recursive fetches
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.googleDocs, url ? url : "https://docs.google.com/feeds/default/private/full", "GET", 
				[["GData-Version", "3.0"],
				 ["Authorization", "GoogleLogin auth=" + token]], null, function(req){
					 
					 if (!files)
						var files = [];
					 
					 var entry = req.responseXML.documentElement.getElementsByTagName("entry");
					 for (var i = 0; i < entry.length; i++) {
						var title = entry[i].getElementsByTagName("title")[0].textContent;
						var id = entry[i].getElementsByTagName("id")[0].textContent;
						var content = entry[i].getElementsByTagName("content")[0];
						var contentURL = content.getAttribute("src");
						var type = content.getAttribute("type");
						//var description = entry[i].getElementsByTagName("description")[0].textContent;
						var urls = entry[i].getElementsByTagName("link");
						var editURL;
						for (var j = 0; j < urls.length; j++){
							if (urls[j].getAttribute("rel") == "edit") {
								editURL = urls[j].getAttribute("href");
							}
							if (urls[j].getAttribute("rel") == "edit") {
								editURL = urls[j].getAttribute("href");
							}
						}
						 files.push({
							title: title,
							id: id,
							contentURL: contentURL,
							type: type,
							//description: description,
							editURL: editURL
						 });
					 }
					 //Is the feed complete?
					 var link = req.responseXML.documentElement.getElementsByTagName("link");
					 for (var i = 0; i < link.length; i++) {
						var nURL = link[i].getAttribute("href");
						if (/start-key/.test(nURL)) {
							this.getList(token, fun, parameters, nURL, files);
							return;
						}
					 }
					 fun.apply(QuickFoxNotes.network.googleDocs, [files].concat(parameters));
				 }, parameters);
		},
		readFile: function (token, url, fun, parameters) {	//url is contentURL
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.googleDocs, url + "&exportFormat=txt&format=txt", "GET",
			 [["GData-Version", "3.0"],
			  ["Authorization", "GoogleLogin auth=" + token]], null, function(req){
				fun.apply(QuickFoxNotes.network.googleDocs, [req.responseText.replace(new RegExp(String.fromCharCode(13, 10), 'g'), '\n')].concat(parameters));
			  }, parameters);
		},
		deleteFile: function (token, url, fun, parameters) {	//url is editURL
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.googleDocs, url, "DELETE",
			 [["GData-Version", "3.0"],
			  ["If-Match", "*"],
			  ["Authorization", "GoogleLogin auth=" + token]], null, function(req){
				fun.apply(QuickFoxNotes.network.googleDocs, parameters);
			  }, parameters);
		},
		rename: function (token, url, new_name, fun, parameters) {	//url is editURL
			var doc = document.implementation.createDocument("", "", null);
			network._makeDoc(doc, doc,
				["entry@http://www.w3.org/2005/Atom",[	//Add namespace to all sub elements too
				 ["category@http://www.w3.org/2005/Atom", ""], 
				 ["title@http://www.w3.org/2005/Atom", new_name]
				]]
			);
			var entry = doc.getElementsByTagName("entry")[0];
			entry.setAttribute("xmlns:docs", "http://schemas.google.com/docs/2007");
			var category = doc.getElementsByTagName("category")[0];
			category.setAttribute("scheme", "http://schemas.google.com/g/2005#kind");
			category.setAttribute("term", "http://schemas.google.com/docs/2007#document");
			
			var serializer = new XMLSerializer();
			doc = serializer.serializeToString(doc);
			
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.googleDocs, url, "PUT",
			 [["GData-Version", "3.0"],
			  ["Content-Type", "application/atom+xml"],
			  ["Content-Length", doc.length],
			  ["If-Match", "*"],
			  ["Authorization", "GoogleLogin auth=" + token]], doc, function(req){
							alert(req.responseText);
				//fun.apply(QuickFoxNotes.network.googleDocs, parameters);
			  }, parameters);
		}
	},
	/**************************Sugarsync***********************************/
	sugarsync: {
		getToken: function(username, password, fun, parameters, errFun) {
			var doc = document.implementation.createDocument("", "", null);
			QuickFoxNotes.network._makeDoc(doc, doc,
				["authRequest", [
					["username", username], 
					["password", password], 
					["accessKeyId", "MTM4NDY2NDEzMDc1NDc1NzY4MDA"], 
					["privateAccessKey", "MGQ4ZGUyNTdhNjlkNDk0Yjg5MmMyOGFiOGUzZTc2YTY"]
				]]
			);
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.sugarsync, "https://api.sugarsync.com/authorization", "POST", [], doc, function(req){
				try {
					var token = req.getAllResponseHeaders().match(/Location: (.*)/m)[1];
					
					fun.apply(QuickFoxNotes.network.sugarsync, [token].concat(parameters));
				}
				catch(e) {
					errFun(req);
				}
			}, [], errFun);
		},
		//Get list of availble PCs
		getWorkspaces: function (token, fun, parameters) {
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.sugarsync, "https://api.sugarsync.com/user", "GET", [["Authorization", token]], null, function(req){
				var workspaces = req.responseXML.documentElement.getElementsByTagName("workspaces")[0].textContent;
				QuickFoxNotes.network._sendReq(QuickFoxNotes.network.sugarsync, workspaces, "GET", [["Authorization", token]], null, function(req){
					var PCs = [];
					var collection = req.responseXML.documentElement.getElementsByTagName("collection");
					for (var i = 0; i < collection.length; i++){
						PCs.push(
						{
							url: collection[i].getElementsByTagName("ref")[0].textContent,
							name: collection[i].getElementsByTagName("displayName")[0].textContent
						});
					}
					fun.apply(QuickFoxNotes.network.sugarsync, [PCs].concat(parameters));
				}, []);
			}, []);
		},
		//Get list of Folders within a URL
		getList: function (token, url, type, fun, parameters) {	//types: "collection" and "file"
			//Get Folders
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.sugarsync, url, "GET", [["Authorization", token]], null, function(req){
				var list = [];
				var collection = req.responseXML.documentElement.getElementsByTagName(type);
				for (var i = 0; i < collection.length; i++){
					list.push(
					{
						url: collection[i].getElementsByTagName("ref")[0].textContent,
						name: collection[i].getElementsByTagName("displayName")[0].textContent
					});
				}
				fun.apply(QuickFoxNotes.network.sugarsync, [list].concat(parameters));
			}, []);
		},
		//Get working URL for files, folders and parent of a URL
		getContents: function (token, url, fun, parameters) {
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.sugarsync, url, "GET", [["Authorization", token]], null, function(req){
				var contents = req.responseXML.documentElement.getElementsByTagName("contents")[0].textContent;
				var files = req.responseXML.documentElement.getElementsByTagName("files")[0].textContent;
				var parent;	//Not all folders have parent!
				try{	//There is no parent url when listing workspaces
					parent = req.responseXML.documentElement.getElementsByTagName("parent")[0].textContent;
				} catch(e){}
				fun.apply(QuickFoxNotes.network.sugarsync, [contents, files, parent].concat(parameters));
			}, []);
		},
		createFile: function (token, url, name, data, fun, parameters, errFun) {
			var doc = document.implementation.createDocument("", "", null);
			QuickFoxNotes.network._makeDoc(doc, doc,
				["file",[
				 ["displayName", name], 
				 ["mediaType", "text/plain"], 
				 ["token", token]
				]]
			);
			
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.sugarsync, url, "POST", [["Authorization", token]], doc, function(req){
				var url = req.getAllResponseHeaders().match(/Location: (.*)/m)[1] + "/data";
				
				QuickFoxNotes.network._sendReq (QuickFoxNotes.network.sugarsync, url, "PUT", [
				 ["Authorization", token],
				 ["Content-Length", data.length],
				 ["Mime-Type", "text/plain"]
				], data, function(req){
					fun.apply(QuickFoxNotes.network.sugarsync, parameters);
				}, [], errFun);
			}, []);
		},
		readFile: function (token, url, fun, parameters) {
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.sugarsync, url, "GET", [["Authorization", token]], null, function(req){
				var fileURL = req.responseXML.documentElement.getElementsByTagName("fileData")[0].textContent;
				QuickFoxNotes.network._sendReq(QuickFoxNotes.network.sugarsync, fileURL, "GET", [["Authorization", token]], null, function(req){
					fun.apply(QuickFoxNotes.network.sugarsync, [req.responseText.replace(new RegExp(String.fromCharCode(13, 10), 'g'), '\n')].concat(parameters));
				}, []);
			
				
			}, []);
		},
		createFolder: function (token, url, name, fun, parameters) {
			var doc = document.implementation.createDocument("", "", null);
			QuickFoxNotes.network._makeDoc(doc, doc,
				["folder",[
				 ["displayName", name], 
				 ["token", token]
				]]
			);
			
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.sugarsync, url, "POST", [["Authorization", token]], doc, function(req){
				var url = req.getAllResponseHeaders().match(/Location: (.*)/m)[1] + "/data";
			
				fun.apply(QuickFoxNotes.network.sugarsync, [url].concat(parameters));
			}, []);
		},
		kill: function(token, url, fun, parameters) {
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.sugarsync, url, "DELETE", [["Authorization", token]], null, function(req){
				fun.apply(QuickFoxNotes.network.sugarsync, parameters);
			}, []);
		},
		rename: function (token, url, new_name, type, fun, parameters) {	//type= file or folder
			var doc = document.implementation.createDocument("", "", null);
			network._makeDoc(doc, doc,
				[type,[
				 ["displayName", new_name], 
				 ["token", token]
				]]
			);
			QuickFoxNotes.network._sendReq(QuickFoxNotes.network.sugarsync, url, "PUT", [["Authorization", token]], doc, function(req){
				fun.apply(QuickFoxNotes.network.sugarsync, parameters);
			}, []);
		}
	},
	/**************************Simplenote***********************************/
	simplenote: {
		token: null,
		username: null,
	
		getToken: function (username, password, fun, parameters, errFun, force) {
			if (this.token && !force) {
				fun.apply(QuickFoxNotes.network.simplenote, [this.token].concat(parameters));
				return;
			}
			var doc = "?" + btoa("email=" + username + "&password=" + password);
		
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.simplenote, "https://simple-note.appspot.com/api/login", "POST", [
			 ["Content-type", "application/x-www-form-urlencoded"],
			 ["Content-length", doc.length],
			 ["Connection", "close"]
			], doc, function(req){
				this.token = req.responseText;
				if (this.token) {
					this.username = username;
					fun.apply(QuickFoxNotes.network.simplenote, [this.token].concat(parameters));
				}
				else
					errFun.apply(QuickFoxNotes.network.simplenote, [req]);
			}, [], errFun);
		},
		
		getList: function (token, fun, parameters, errFun, mark, IDs) {	//mark is used for several request
			var url = "https://simple-note.appspot.com/api2/index?auth=" + token + "&email=" + this.username + (mark ? "&mark="+mark : "") + "&length=100";
		
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.simplenote, url, "GET", [], null, function(req){
				if (!IDs)
					var IDs = [];

				var list = qfnServices.nativeJSON.decode (req.responseText);
				for (var i = 0; i < list.count; i++) {
					if (list.data[i].deleted == 1)	//Do not get info of deleted note
						continue;
					IDs.push({
						key: list.data[i].key,
						tags: list.data[i].tags
					});
				}
				if (list.mark) {
					this.getList(token, fun, parameters, errFun, list.mark, IDs);
					return;
				}
				else
					fun.apply(QuickFoxNotes.network.simplenote, [IDs].concat(parameters));
			}, [], errFun);
		},
		
		getContent: function (token, key, fun, parameters, errFun) {	//Returns null if the note is deleted
			var url = "https://simple-note.appspot.com/api2/data/" + key + "?auth=" + token + '&email=' + this.username;
			
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.simplenote, url, "GET", [], null, function(req){
				var list = qfnServices.nativeJSON.decode (req.responseText);
				fun.apply(QuickFoxNotes.network.simplenote, [(list.deleted == 1 ? null : list.content)].concat(parameters));
			}, [], errFun);
		},
		
		write: function (token, key, data, fun, parameters, errFun) {
			var url = "https://simple-note.appspot.com/api2/data" + (key ? "/" + key : "") + "?auth=" + token + "&email=" + this.username;
			var doc = qfnServices.nativeJSON.encode({content:data});
			
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.simplenote, url, "POST", [["Content-Type", "text/plain;charset=UTF-8"]], doc, function(req){
				fun.apply(QuickFoxNotes.network.simplenote, parameters);
			}, [], errFun);
		},
		kill: function (token, key, fun, parameters, errFun) {
			var url = "https://simple-note.appspot.com/api2/data/" + key + "?auth=" + token + "&email=" + this.username;
			var doc = qfnServices.nativeJSON.encode({"deleted": 1});
			
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.simplenote, url, "POST", [["Content-Type", "text/plain;charset=UTF-8"]], doc, function(req){
				fun.apply(QuickFoxNotes.network.simplenote, parameters);
			}, [], errFun);
		},
		getTags: function(token, fun, parameters, errFun, mark, IDs) {
			var url = "https://simple-note.appspot.com/api2/tags" + "?auth=" + token + "&email=" + this.username + (mark ? "&mark="+mark : "") + "&length=100";
		
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.simplenote, url, "GET", [], null, function(req){
				if (!IDs)
					var IDs = [];

				var list = qfnServices.nativeJSON.decode (req.responseText);
				var tags = list.tags;
				
				for (var i = 0; i < list.count; i++)
					IDs.push(tags[i].name);

				if (list.mark) {
					this.getTags(token, fun, parameters, errFun, list.mark, IDs);
					return;
				}
				else
					fun.apply(QuickFoxNotes.network.simplenote, [IDs].concat(parameters));
			}, [], errFun);
		}
	},
	/**************************Evernote***********************************/
	evernote: {
		token: null,
		shardId: null,
		
		username: null,
		password: null,
		
		
		getToken: function (username, password, fun, parameters, errFun, force) {
			if (this.token && !force) {
				fun.apply(QuickFoxNotes.network.evernote, [this.token].concat(parameters));
				return;
			}
			
			var doc = "task=getToken&username=" + username + "&password=" + btoa(password);
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.evernote, "http://quickfoxnotes.inbasic.net/evernote/evernote.php", "POST", [
			 ["Content-type", "application/x-www-form-urlencoded"],
			 ["Content-length", doc.length],
			 ["Connection", "close"]
			], doc, function(req){
				try {
					var list = qfnServices.nativeJSON.decode (req.responseText);
					this.shardId = list.shardId; 
					this.token = list.token;
					
					fun.apply(QuickFoxNotes.network.evernote, [this.token].concat(parameters))
				}
				catch(e) {
					errFun.apply(QuickFoxNotes.network.evernote, [req]);
				}
			}, [], errFun);
		},
		listNotebooks: function (token, fun, parameters, errFun) {
			var doc = "task=listNotebooks&token=" + token + "&shardId=" + this.shardId;
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.evernote, "http://quickfoxnotes.inbasic.net/evernote/evernote.php", "POST", [
			 ["Content-type", "application/x-www-form-urlencoded"],
			 ["Content-length", doc.length],
			 ["Connection", "close"]
			], doc, function(req){
				try {
					var notebooks = qfnServices.nativeJSON.decode (req.responseText);

					fun.apply(QuickFoxNotes.network.evernote, [notebooks].concat(parameters))
				}
				catch(e) {
					errFun.apply(QuickFoxNotes.network.evernote, [req]);
				}
			}, [], errFun);
		},
		findNotes: function (token, notebookGuid, fun, parameters, errFun) {
			var doc = "task=findNotes&token=" + token + "&shardId=" + this.shardId + "&notebookGuid=" + notebookGuid;
			
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.evernote, "http://quickfoxnotes.inbasic.net/evernote/evernote.php", "POST", [
			 ["Content-type", "application/x-www-form-urlencoded"],
			 ["Content-length", doc.length],
			 ["Connection", "close"]
			], doc, function(req){
				try {
					var notes = qfnServices.nativeJSON.decode (req.responseText);

					fun.apply(QuickFoxNotes.network.evernote, [notes].concat(parameters))
				}
				catch(e) {
					errFun.apply(QuickFoxNotes.network.evernote, [req]);
				}
			}, [], errFun);
		},
		getNoteContent: function (token, guid, fun, parameters, errFun) {
			var doc = "task=getNoteContent&token=" + token + "&shardId=" + this.shardId + "&guid=" + guid;
			
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.evernote, "http://quickfoxnotes.inbasic.net/evernote/evernote.php", "POST", [
			 ["Content-type", "application/x-www-form-urlencoded"],
			 ["Content-length", doc.length],
			 ["Connection", "close"]
			], doc, function(req){
				fun.apply(QuickFoxNotes.network.evernote, [req.responseText].concat(parameters))
			}, [], errFun);
		},
		createNote: function (token, notebookGuid, title, content, fun, parameters, errFun) {
			function removeHTMLSpecialChars (text) {
				return text.replace(/&/g, "&amp;").replace(/>/g, "&gt;").replace(/</g, "&lt;").replace(/"/g, "&quot;").replace(/'/g, "&#039;");
			}
			title = encodeURIComponent(title);
			content = removeHTMLSpecialChars(content);
			//content = "<p>" + content.split("\n").join("</p>\n<p>") + "</p>";
			content = "<pre>" + content + "</pre>";
			content = encodeURIComponent(content);
		
			var doc = "task=createNote&token=" + token + "&shardId=" + this.shardId + "&notebookGuid=" + notebookGuid + "&title=" + title + "&content=" + content;
			
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.evernote, "http://quickfoxnotes.inbasic.net/evernote/evernote.php", "POST", [
			 ["Content-type", "application/x-www-form-urlencoded"],
			 ["Content-length", doc.length],
			 ["Connection", "close"]
			], doc, function(req){
				fun.apply(QuickFoxNotes.network.evernote, [req.responseText].concat(parameters))	//responseText is guid
			}, [], errFun);
		},
		createNotebook: function (token, name, fun, parameters, errFun) {
			name = encodeURIComponent(name);
			var doc = "task=createNotebook&token=" + token + "&shardId=" + this.shardId + "&name=" + name;
			
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.evernote, "http://quickfoxnotes.inbasic.net/evernote/evernote.php", "POST", [
			 ["Content-type", "application/x-www-form-urlencoded"],
			 ["Content-length", doc.length],
			 ["Connection", "close"]
			], doc, function(req){
				fun.apply(QuickFoxNotes.network.evernote, [req.responseText].concat(parameters))	//responseText is guid
			}, [], errFun);
		},
		deleteNote: function (token, guid, fun, parameters, errFun) {
			var doc = "task=deleteNote&token=" + token + "&shardId=" + this.shardId + "&guid=" + guid;
			
			QuickFoxNotes.network._sendReq (QuickFoxNotes.network.evernote, "http://quickfoxnotes.inbasic.net/evernote/evernote.php", "POST", [
			 ["Content-type", "application/x-www-form-urlencoded"],
			 ["Content-length", doc.length],
			 ["Connection", "close"]
			], doc, function(req){
				fun.apply(QuickFoxNotes.network.evernote, parameters);
			}, [], errFun);
		},
	},
	/**************************Local***********************************/
	local: {
		file: null,
		
		getToken: function (fun, parameters, errFun) {
			this.file = Components.classes["@mozilla.org/file/local;1"].
					   createInstance(Components.interfaces.nsILocalFile);

			fun.apply(QuickFoxNotes.network.local, ["local_token"].concat(parameters));
		},
		list: function (path, fun, parameters, errFun) {
			if(path == "/" && /WIN/.test(qfnServices.appInfo.OS))
				path = "\\\\.";
			try {
				this.file.initWithPath(path);
				var folderEnum = this.file.directoryEntries;
				var drives = [];
				var files = [];
				while (folderEnum.hasMoreElements()) {
					var f = folderEnum.getNext().
					QueryInterface(Components.interfaces.nsILocalFile);
					
					var path = f.path;
					var name = f.leafName;
					var isFolder = false;
					try{isFolder = f.isDirectory();}catch(e){};
					if(isFolder)
						drives.push({
							name: name,
							path: path,
							isFolder: isFolder
						});
					else
						files.push({
							name: name,
							path: path,
							isFolder: isFolder
						});
				}
				fun.apply(QuickFoxNotes.network.local, [files.concat(drives)].concat(parameters));
			}
			catch(e){
				errFun.apply(QuickFoxNotes.network.local, [e.message]);
			}
		},
		createFolder: function(path, name, fun, parameters, errFun){
			try {
				this.file.initWithPath(path);
				this.file.append(name);
				if( !this.file.exists() || !this.file.isDirectory() )  // if it doesn't exist, create
					this.file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
				fun.apply(QuickFoxNotes.network.local, parameters);
			}
			catch(e){
				errFun.apply(QuickFoxNotes.network.local, [e.message]);
			}
		},
		readFile: function(path, fun, parameters, errFun){
			try {
				this.file.initWithPath(path);
				QuickFoxNotes.importText(this.file);
				
				fun.apply(QuickFoxNotes.network.local, parameters);
			}
			catch(e){
				errFun.apply(QuickFoxNotes.network.local, [e.message]);
			}
		},
		writeFile: function(path, title, content, fun, parameters, errFun){
			Components.utils.import("resource://gre/modules/NetUtil.jsm");
			Components.utils.import("resource://gre/modules/FileUtils.jsm");

			try {
				this.file.initWithPath(path);
				this.file.append(title + ".txt");
				
				var i = 1;
				while (this.file.exists()) {
					this.file.initWithPath(path);
					this.file.append(title + " (" + i + ").txt");
					i += 1;
				}

				var ostream = FileUtils.openSafeFileOutputStream(this.file)

				var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
								createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
				converter.charset = "UTF-8";
				var istream = converter.convertToInputStream(content);


				NetUtil.asyncCopy(istream, ostream, function(status) {
					if (!Components.isSuccessCode(status)) {
						errFun.apply(QuickFoxNotes.network.local, [status]);
						return;
					}
					fun.apply(QuickFoxNotes.network.local, parameters);
				})
			}
			catch(e) {
				errFun.apply(QuickFoxNotes.network.local, [e.message]);
			};
		},
		kill: function(path, isFolder, fun, parameters, errFun){
			try {
				this.file.initWithPath(path);
				this.file.remove(isFolder);
				
				fun.apply(QuickFoxNotes.network.local, parameters);
			}
			catch(e) {
				errFun.apply(QuickFoxNotes.network.local, [e.message]);
			};
		}
	}
}